'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _simplSchema = require('simpl-schema');

var _simplSchema2 = _interopRequireDefault(_simplSchema);

var _utils = require('../utils');

var _utils2 = _interopRequireDefault(_utils);

var _reactFormHandler = require('../react-form-handler');

var _reactFormHandler2 = _interopRequireDefault(_reactFormHandler);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Form = function (_Component) {
    _inherits(Form, _Component);

    function Form() {
        _classCallCheck(this, Form);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Form).call(this));

        _this._onSubmit = _this._onSubmit.bind(_this);
        _this.submit = _this.submit.bind(_this);

        _this.state = {
            errors: false
        };
        return _this;
    }

    // Assuming that the schema won't get changed


    _createClass(Form, [{
        key: 'getChildContext',
        value: function getChildContext() {
            return { schema: this.props.schema };
        }
    }, {
        key: 'componentWillMount',
        value: function componentWillMount() {
            _reactFormHandler2.default.initializeForm(this.props.id);
        }
    }, {
        key: 'focusInput',
        value: function focusInput(name) {
            // Set focus to a form input by name
            this.refs[name].refs.input.focus();
        }
    }, {
        key: 'submit',
        value: function submit() {
            this._onSubmit();
        }
    }, {
        key: 'getDoc',
        value: function getDoc() {
            return _reactFormHandler2.default.getFormDoc(this.props.id);
        }
    }, {
        key: '_onSubmit',
        value: function _onSubmit(event) {
            // Event is not defined if onSubmit is called pragmatically
            if (event) {
                event.preventDefault();
            }
            var schema = this.props.schema;
            var formToDoc = _reactFormHandler2.default[this.props.id] && _reactFormHandler2.default[this.props.id].formToDoc;

            var doc = schema.clean(_.reduce(this.refs.form.querySelectorAll('[name]'), function (doc, field) {
                doc[field.name] = _reactFormHandler2.default.getFieldValue(field);
                return doc;
            }, {}));

            var validationContext = schema.newContext();

            // Temporarily remove the doc's ignored fields
            // so an error isn't thrown when validating
            var ignoredFields = {};
            _.each(_reactFormHandler2.default.ignoreFields, function (field) {
                ignoredFields[field] = doc[field];
                delete doc[field];
            });

            // Remove empty strings
            doc = _utils2.default.cleanNulls(doc);
            doc = formToDoc ? formToDoc(doc) : doc;

            if (!validationContext.validate(doc)) {
                var errors = {};

                _.each(validationContext.validationErrors(), function (error) {
                    errors[error.name] = error.type;
                });

                this.setState({ errors: errors });

                if (_reactFormHandler2.default.debug) {
                    console.error("React form handler validation errors", errors);
                }
            } else {
                this.setState({ errors: false });
                _reactFormHandler2.default.submitForm(this.props.id);

                if (this.props.onSubmit) {

                    // Restore ignored fields
                    if (ignoredFields) {
                        _.each(ignoredFields, function (value, key) {
                            if (value) {
                                doc[key] = value;
                            }
                        });
                    }

                    this.props.onSubmit(doc);
                }

                if (this.props.resetOnSubmit !== false) {
                    this.refs.form.reset();
                }
            }
        }
    }, {
        key: '_renderChildren',
        value: function _renderChildren() {

            if (this.props.doc) {
                _reactFormHandler2.default.setFormDoc(this.props.id, this.props.doc);
            }

            var that = this;
            var formDoc = _reactFormHandler2.default.getFormDoc(this.props.id);

            if (this.props.children) {
                return React.Children.map(this.props.children, function (child) {
                    var _this2 = this;

                    if (child.props && child.props.name) {
                        var schemaObject = this.props.schema._schema[child.props.name];

                        if (!!schemaObject) {
                            var newChildProps = _.clone(child.props);
                            newChildProps.ref = child.props.name;
                            newChildProps.formId = that.props.id;
                            newChildProps.formSchema = this.props.schema;

                            if (!formDoc[child.props.name]) {
                                if (child.props.defaultValue) {
                                    newChildProps.defaultValue = child.props.defaultValue;
                                } else if (schemaObject.defaultValue) {
                                    newChildProps.defaultValue = schemaObject.defaultValue;
                                }
                            } else {
                                newChildProps.defaultValue = formDoc[child.props.name];
                            }

                            if (schemaObject.type === Number) {
                                newChildProps._valueType = "number";
                            }

                            if (newChildProps.label) {
                                newChildProps.label = _reactFormHandler2.default.i18n ? this.context.intl.formatMessage({ id: newChildProps.label }) : newChildProps.label;
                            } else if (schemaObject.label) {
                                newChildProps.label = _reactFormHandler2.default.i18n ? this.context.intl.formatMessage({ id: schemaObject.label }) : schemaObject.label;
                            }

                            if (schemaObject.placeholder || newChildProps.placeholder) {
                                var placeholder = newChildProps.placeholder || schemaObject.placeholder;
                                newChildProps.placeholder = _reactFormHandler2.default.i18n ? this.context.intl.formatMessage({ id: placeholder }) : placeholder;
                            }

                            if (schemaObject.allowedValues) {
                                newChildProps.allowedValues = schemaObject.allowedValues;
                            }

                            if (schemaObject.min) {
                                newChildProps.min = schemaObject.min;
                            }

                            if (schemaObject.max) {
                                newChildProps.max = schemaObject.max;
                            }

                            if (newChildProps.options && newChildProps.i18nOptions) {
                                _.each(newChildProps.options, function (option) {
                                    option.label = _this2.context.intl.formatMessage({ id: option.label });
                                });
                            }

                            if (typeof this.props.schema._schema[child.props.name + ".$"] !== 'undefined') {
                                var itemSchemaObject = this.props.schema._schema[child.props.name + ".$"];
                                if (itemSchemaObject.allowedValues) {
                                    newChildProps.allowedValues = itemSchemaObject.allowedValues;
                                }
                            }

                            var errors = this.state.errors || this.props.errors;

                            if (errors && errors[child.props.name]) {
                                newChildProps.errorText = _reactFormHandler2.default.i18n ? this.context.intl.formatMessage({ id: 'errors.' + errors[child.props.name] }) : errors[child.props.name];
                                newChildProps.error = true;
                            } else if (!child.props.error) {
                                newChildProps.errorText = '';
                                newChildProps.error = false;
                            }

                            return React.cloneElement(child, newChildProps);
                        } else {
                            throw new Meteor.Error("input_schema_missing", "There is no schema defined for the input " + child.props.name);
                        }
                    } else if (child.props.label) {
                        var newChildProps = _.clone(child.props);

                        if (_reactFormHandler2.default.i18n) {
                            newChildProps.label = this.context.intl.formatMessage({ id: newChildProps.label }, newChildProps.values);
                        }

                        return React.cloneElement(child, newChildProps);
                    } else {
                        return child;
                    }
                }.bind(this));
            }
        }
    }, {
        key: 'render',
        value: function render() {

            var className = "";

            if (this.props.className) {
                className += " " + this.props.className;
            }

            if (_reactFormHandler2.default.formClass) {
                className += " " + _reactFormHandler2.default.formClass;
            }

            return React.createElement(
                'form',
                { className: className, ref: 'form', id: this.props.id, onSubmit: this._onSubmit },
                this._renderChildren(),
                React.createElement('div', { style: { clear: "both" } })
            );
        }
    }]);

    return Form;
}(_react.Component);

;

Form.propTypes = {
    schema: _react.PropTypes.instanceOf(_simplSchema2.default).isRequired,
    id: _react.PropTypes.string,
    onSubmit: _react.PropTypes.func,
    resetOnSubmit: _react.PropTypes.bool
};

Form.childContextTypes = {
    schema: _react.PropTypes.instanceOf(_simplSchema2.default)
};

Form.contextTypes = {
    intl: _react.PropTypes.object
};

exports.default = Form;